package ihm.compilateur;

import ihm.runtimeIhm.DebugWindow;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Constructor;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import util.MyLoader;
import util.PipeStream;
import antlr.RecognitionException;
import antlr.TokenStreamException;

import compilateur.CompilationException;
import compilateur.L;
import compilateur.P;
import compilateur.cspLangage.Programme;

import csp_runtime.CspThread;

/**
 * 
 * @author Benjamin LE BOZEC
 *
 */
public class Actions implements ActionListener {

	MainWindow main;
	private boolean compilationOk;
	
	public Actions(MainWindow main) {
		this.main = main;
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("parcourirJavac"))
			parcourirJavac ();
		if (e.getActionCommand().equals("parcourirOutputDir"))
			parcourirOutputDir ();
		if (e.getActionCommand().equals("parcourirCspFile"))
			parcourirCspFile ();
		if (e.getActionCommand().equals("compiler"))
			compilerRunnable();
		if (e.getActionCommand().equals("executer"))
			executerRunnable();
		if (e.getActionCommand().equals("apropos"))
			aPropos();
	}

	private void aPropos() {
		JOptionPane.showMessageDialog(main, "CTJ - Compilateur de programmes CSP en JAVA\nBenjamin LE BOZEC (benjamin_lebozec@hotmail.fr)\nversion1.0 - 2005\n\n(Utilisation de ANTLR - www.antlr.org - pour l'analyse lexicale et syntaxique)");
	}

	private void runTask (Runnable task) {
		new Thread (task).start();
	}
	
	public void executerRunnable () {
		runTask(new Runnable () {
			public void run() { executer (); }
		});
		
	}
	
	private void executer() {
		
		MyLoader loader = new MyLoader ();				
		
		String packageName = main.getOutputDir ();
		
		if (!checkPackageName()) {
			JOptionPane.showMessageDialog(main, "Le nom du package n'est pas valide (un nom uniquement compos de lettres est souhait)");
			return;
		}
		
		CspThread thread;
		try {
			loader.load(packageName);
		
			Class<?> toRun = loader.loadClass(packageName + "." + "MAIN");
			Constructor<?> constr = toRun.getConstructor(new Class[]{String.class});
			thread = (CspThread) constr.newInstance(new Object[]{"MAIN"});
		} catch (Exception e) {
			JOptionPane.showMessageDialog(main, "Impossible d'excuter le programme");
			return;
		}
		main.setVisible(false);
		CspThread.flushEverything ();
		new DebugWindow (main, thread);
	}

	public void compilerRunnable () {
		runTask(new Runnable () {
			public void run() { 
				main.setBoutonsEnabled(false);
				compiler ();
				main.setBoutonsEnabled(true);
			}
		});
		
	}
	
	private void compiler() {
		
		/*
		 * verification de la validit du fichier csp
		 */ 
		if (main.getCspFile().equals("")) {
			JOptionPane.showMessageDialog(main, "Veuillez d'abord choisir un fichier CSP  compiler");
			return;
		}
		
		/*
		 * verification de la validit du javac
		 */
		if (!new File(main.getJavac()).exists()) {
			JOptionPane.showMessageDialog(main, "Le compilateur javac selectionn n'est pas valide");
			return;
		}
		
		/*
		 * Vrification du nom du package
		 */
		if (!checkPackageName ()) {
			JOptionPane.showMessageDialog(main, "Le nom du package n'est pas valide (un nom uniquement compos de lettres est souhait");
			return;
		}
			
		
		/*
		 * clean
		 */
		clean(main.getOutputDir().replaceAll("\\.", ""+File.separatorChar));
		
		/*
		 * ouverture du fichier
		 */ 
		FileInputStream fis;
		try {
			fis = new FileInputStream(main.getCspFile());
		} catch (FileNotFoundException e) {
			main.getMsgArea().appendln("Fichier " + main.getCspFile() + " non valide");
			return;
		}
		
		/*
		 * lancement de l'analyse syntaxique
		 */ 
		compilationOk = true;
		main.getMsgArea().clean ();
		main.getMsgArea().appendln ("Analyse syntaxique ...");
		L lexer = new L (fis);
		lexer.setTabSize(1);
		P parser = new P (lexer) {
			public void reportError(RecognitionException ex) {
				main.getMsgArea().appendln (ex.toString());
				compilationOk = false;
			}
			public void reportError(String str) {
				main.getMsgArea().appendln("Error : " + str);
				compilationOk = false;
			}
			public void reportWarning(String str) {
				main.getMsgArea().appendln ("Warning : " + str);
				compilationOk = false;
			}
		};
		Programme prog;
		try {
			prog = parser.programme();
		} catch (RecognitionException e) {
			main.getMsgArea().append(e);
			return;
		} catch (TokenStreamException e) {
			main.getMsgArea().append(e);
			return;
		} catch (NullPointerException e) {
			if (compilationOk) throw e;
			return;
		}
		if (!compilationOk) return; 
		
		/*
		 * lancement de la compilation CSP->JAVA
		 */ 
		main.getMsgArea().appendln("Compilation CSP->JAVA...");
		prog.preCompile ();
		
		try {
			prog.compile(main.getOutputDir());
		} catch (CompilationException e) {	
			main.getMsgArea().append(e);
			return;
		}
		
		/*
		 *  compilation JAVA
		 */
		main.getMsgArea().appendln("Compilation JAVA...");
		
		try {
			Process javacProcess = Runtime.getRuntime().exec (
					new String[]{main.getJavac(), main.getOutputDir().replaceAll("\\.", ""+File.separatorChar) + File.separatorChar + "MAIN.java"});
			new PipeStream (javacProcess.getErrorStream(), true) {
				public void reportError(IOException e) {
					JOptionPane.showMessageDialog(main, "Problme d'e/s lors de l'appel  " + main.getJavac());
				}
				public void write(String str) {
					main.getMsgArea().append(str);
				}
			};
			new PipeStream (javacProcess.getInputStream(), true) {
				public void reportError(IOException e) {
					JOptionPane.showMessageDialog(main, "Problme d'e/s lors de l'appel  " + main.getJavac());
				}
				public void write(String str) {
					main.getMsgArea().append(str);
				}
			};
			if (javacProcess.waitFor()==0) main.getMsgArea().append ("compilation OK"); 			
		} catch (IOException e) {
			JOptionPane.showMessageDialog(main, "Problme d'e/s lors de l'appel  " + main.getJavac());
			return;
		} catch (InterruptedException e) {
			return;
		}
		
	}

	private boolean checkPackageName() {
		String packageName = main.getOutputDir();
		if (packageName.matches("[a-zA-Z]([a-zA-Z]|[0-9])+"))
			return true;
		else
			return false;
	}

	private void parcourirCspFile() {
		JFileChooser chooser = new JFileChooser ();
		chooser.setCurrentDirectory(new File(main.getCspFile()));
		
		chooser.setFileFilter(new FileFilter () {
			public String getDescription() {
				return "programmes CSP";
			}
		
			public boolean accept(File f) {
				if (f.getName().endsWith(".csp")) return true;
				if (f.isDirectory()) return true;
				return false;
			}
		
		});
		
		if (chooser.showOpenDialog(main)==JFileChooser.APPROVE_OPTION)
			main.setCspFile (chooser.getSelectedFile().getAbsolutePath());
	}

	private void parcourirOutputDir() {
		JFileChooser chooser = new JFileChooser ();
		chooser.setCurrentDirectory(new File(main.getOutputDir()));
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		
		if (chooser.showOpenDialog(main)==JFileChooser.APPROVE_OPTION)
			main.setOutputDir(chooser.getSelectedFile().getAbsolutePath());
		
	}

	private void parcourirJavac() {
		JFileChooser chooser = new JFileChooser ();
		chooser.setCurrentDirectory(new File(main.getJavac()));
		
		if (chooser.showOpenDialog(main)==JFileChooser.APPROVE_OPTION)
			main.setJavac (chooser.getSelectedFile().getAbsolutePath());
	}

	public static void clean (String dir) {
		
		File dirFile = new File(dir);
		if (dirFile.exists()) {
			File [] list = new File (dir).listFiles();
			for (File f : list)
				f.delete();
		}
	}

}
