package ihm.compilateur;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import util.GBFactory;
import util.saveObject.InputSaveObject;
import util.saveObject.OutputSaveObject;
import util.saveObject.SavableColor;
import util.saveObject.SaveObjectException;

public class MainWindow extends JFrame {

	private static final String title = "Compilateur CSP->JAVA";
	private static final String defaultOutputDir = "outputDir";
	private static final String saveFile = "cspConfig.xml";
	
	private Color  debugColor = Color.red;
	
	private Actions actions = new Actions(this);
	
	private JPanel mainPanel = new JPanel ();
	private JTextField cspFileField = new JTextField (50);
	private JTextComponent outputDirField = new JTextField (50);
	private JTextComponent javacField = new JTextField (50);
	private MsgArea msgArea = new MsgArea ();
	private JButton boutonCompiler = new JButton ("Compiler");
	private JButton boutonRun = new JButton ("run !!");
	private JButton boutonAPropos = new JButton ("A propos");
	
	public MainWindow () {
		super (title);
		
		/* chargement des prfrences */
		loadPreferences();
		
		mainPanel.setLayout (new GridBagLayout());
		JPanel panelTop = new JPanel(new GridBagLayout());
		
		/* fichier csp*/
		//JPanel panel = new JPanel (new FlowLayout(FlowLayout.RIGHT));
		panelTop.add (new JLabel ("programme csp :"),
				GBFactory.getCst(0, 0, 1, 1, 0, 0, new Insets(3,3,3,3), GridBagConstraints.EAST,  GridBagConstraints.NONE));
		panelTop.add (cspFileField,
				GBFactory.getCst(1, 0, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		JButton bouton = new JButton ("...");
		bouton.setActionCommand("parcourirCspFile");
		bouton.addActionListener(actions);
		panelTop.add(bouton,
				GBFactory.getCst(2, 0, 1, 1, 0, 0, new Insets(4, 4, 4, 4), GridBagConstraints.CENTER, GridBagConstraints.NONE));
		
		/* javac */
		//panel = new JPanel (new FlowLayout(FlowLayout.RIGHT));
		panelTop.add (new JLabel ("javac (>=1.5):"),
				GBFactory.getCst(0, 1, 1, 1, 0, 0, new Insets(2,2,2,2), GridBagConstraints.EAST, GridBagConstraints.NONE));
		panelTop.add (javacField,
				GBFactory.getCst(1, 1, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		bouton = new JButton ("...");
		bouton.setActionCommand("parcourirJavac");
		bouton.addActionListener(actions);
		panelTop.add(bouton,
				GBFactory.getCst(2, 1, 1, 1, 0, 0, new Insets(4,4, 4, 4), GridBagConstraints.CENTER, GridBagConstraints.NONE));
		
		/* outputDir */ 
		panelTop.add (new JLabel ("package cible :"),
				GBFactory.getCst(0, 2, 1, 1, 0, 0, new Insets(3,3,3,3), GridBagConstraints.EAST,  GridBagConstraints.NONE));
		panelTop.add (outputDirField,
				GBFactory.getCst(1, 2, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
				
		/* boutons de compilation et d'execution */
		JPanel panel = new JPanel (new FlowLayout(FlowLayout.RIGHT));
		
		boutonAPropos.setActionCommand("apropos");
		boutonAPropos.addActionListener(actions);
		panel.add (boutonAPropos);
		
		boutonCompiler.setActionCommand("compiler");
		boutonCompiler.addActionListener (actions);
		panel.add (boutonCompiler);
		
		boutonRun.setActionCommand("executer");
		boutonRun.addActionListener(actions);
		panel.add (boutonRun);
		panelTop.add(panel,
				GBFactory.getCst(1, 3, 1, 1, 0, 0, new Insets(3,3,3,3), GridBagConstraints.EAST,  GridBagConstraints.NONE));
		
		/* general */
		mainPanel.add (panelTop, 
				GBFactory.getCst(0, 0, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		mainPanel.add (msgArea,
				GBFactory.getCst(0, 1, 1, 1, 100, 100, GridBagConstraints.BOTH));
		JLabel label = new JLabel ("Benjamin LE BOZEC - 2005", JLabel.RIGHT);
		label.setFont (new Font("Arial", 0, 12));
		mainPanel.add (label,
				GBFactory.getCst(0, 2, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		
		addWindowListener (new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				closeAppli ();
			}
		});
		
		setContentPane (mainPanel);
		pack ();
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setMinimumSize(new Dimension(750, 400));
		setSize(750, 400);
		setLocation ((screen.width - getWidth())/2, (screen.height - getHeight())/2);
		setVisible (true);
	}
	
	private void loadPreferences() {
		if (new File (saveFile).exists()) {
			try {
				FileInputStream fis = new FileInputStream (saveFile);
				InputSaveObject sObj = new InputSaveObject (fis);
				
				outputDirField.setText((String) sObj.getNext("outputDir"));
				javacField.setText((String) sObj.getNext("javac"));
				cspFileField.setText((String) sObj.getNext("cspFile"));
				debugColor = ((SavableColor) sObj.getNext ("debugColor")).getColor();
				
				fis.close ();
				return;
			} catch (IOException e) {
			} catch (SaveObjectException e) {
			}
		}
		setOutputDir(defaultOutputDir);
	}

	public void savePreferences () {
		try {
			FileOutputStream fos = new FileOutputStream (saveFile);
			OutputSaveObject sObj = new OutputSaveObject ();
			
			sObj.add("outputDir", outputDirField.getText());
			sObj.add("javac", javacField.getText());
			sObj.add("cspFile", cspFileField.getText());
			sObj.add("debugColor", new SavableColor(debugColor));
						
			sObj.saveInto(fos);
			fos.close ();
			return;
		} 
		catch (IOException e) { } 
		catch (SaveObjectException e) { System.out.println (e);}
		JOptionPane.showMessageDialog(this, "Problme lors de l'enregistrement des prfrences");
	}
	
	public void closeAppli () {
		savePreferences();
		System.exit(0);
	}
	
	public String getCspFile () { return cspFileField.getText(); }
	
	public void setCspFile (String str) { cspFileField.setText(str); }
	
	public String getJavac () { return javacField.getText (); }
	
	public void setJavac (String str) { javacField.setText (str); }
	
	public String getOutputDir () { return outputDirField.getText (); }
	
	public void setOutputDir (String str) { outputDirField.setText (str); }
	
	public MsgArea getMsgArea () { return msgArea; }

	public void setBoutonsEnabled (boolean v) {
		boutonCompiler.setEnabled(v);
		boutonRun.setEnabled(v);
	}
	
	public void setDebugColor(Color c) {
		debugColor = c;
	}
	
	public Color getDebugColor () {
		return debugColor;
	}

	/**
	 * MAIN
	 * 
	 * @param args
	 * @throws FileNotFoundException 
	 */
	public static void main (String args[]) {
		JFrame.setDefaultLookAndFeelDecorated(true);
		try {
			System.setErr(new PrintStream(new FileOutputStream("err.log", true)));
		} catch (FileNotFoundException e) {
		}
		JDialog.setDefaultLookAndFeelDecorated(true);
		new MainWindow ();
	}
	
}
