package ihm.runtimeIhm;

import java.awt.Font;
import java.util.Collection;
import java.util.Collections;
import java.util.Vector;

import javax.swing.JInternalFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import csp_runtime.CspThread;
import csp_runtime.PrintListener;
import csp_runtime.ValidableListener;

public class ConsoleFrame extends JInternalFrame{
	
	public static final Font defaultFont = new Font ("courier new", 0, 12);
	
	private Vector<CspThread> threads = new Vector<CspThread>();
	private JScrollPane scroll;
	private JTextArea area = new JTextArea ();

	private boolean modeGeneral = false;
	private Collection<ValidableListener> listeners = Collections.synchronizedCollection(new Vector<ValidableListener> ());
	
	public ConsoleFrame () {
		super ("", true, true, true, true);
		
		area = new JTextArea ();
		scroll = new JScrollPane (area);
		
		area.setFont (defaultFont);
		
		setContentPane(scroll);
		setTitle ("Console ");
		setVisible (true);
		area.setEditable(false);
		
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		
		addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent e) {
				if (!modeGeneral) {
					for (ValidableListener listener : listeners)
						listener.valid.set(false);
				}
				setVisible (false);
			}
		});
	}
	
	public void setThread (CspThread thread) {
		area.setText (thread.getConsoleBuffer());
		
		addThread (thread);	
	}
	
	public void addThread(CspThread thread) {
		threads.add (thread);
		
		if (!modeGeneral) setTitle(getTitle() + " " + thread.getCompleteName());
				
		thread.addPrintListener(new PrintListener() {
			public void print(String str) {
				if (valid.get()) {
					try {
					area.append(str);
					scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
					} catch (Exception e) {
						
					}
				}
			}
		});
	}

	public void setThreads (CspThread ... threads) {
		area.setText ("");
		
		for (CspThread thread : threads)
			addThread (thread);			
	}
	
	public void setModeGeneral (boolean b) { 
		modeGeneral  = b;
		if (b) setTitle ("Console gnrale");
	}
}
