/*
 * Created on 27 fvr. 2005
 *
 */
package util.saveObject;

import java.util.*;

/**
 * Cette classe permet la sauvegarde de la classe Map, et plus particulierement
 * la classe HashMap.
 * 
 * @author Benjamin LE BOZEC
 *
 */
public class SavableMap implements Savable {

	SavableMapEntry [] tab;

	/**
	 * Constructeur par dfaut
	 *
	 */
	public SavableMap () {
		
	}
	
	/**
	 * @param map Map a sauvegarder
	 */
	public SavableMap (Map map) {
		Set items = map.entrySet();
		Iterator iter = items.iterator();
		
		tab = new SavableMapEntry [items.size()]; 
		for (int i=0;iter.hasNext();i++)
			tab[i] = new SavableMapEntry ((Map.Entry) iter.next());
	}

	/**
	 * Permet de rcuperer une classe HashMap de la sauvegarde faite
	 * @param initialSize la taille initiale de la HashMap
	 * @return la map ainsi cre
	 */
	public HashMap getHashMap (int initialSize) {
		HashMap map = new HashMap (initialSize);
		for (int i=0;i<tab.length;i++)
			map.put(tab[i].getKey(), tab[i].getValue());
		
		return map;		
	}
	
	/* (non-Javadoc)
	 * @see think.util.Savable#saveInto(think.util.OutputSaveObject)
	 */
	public void saveInto(OutputSaveObject sObj) throws SaveObjectException {
		sObj.add("tab", tab);
	}

	/* (non-Javadoc)
	 * @see think.util.Savable#loadFrom(think.util.InputSaveObject)
	 */
	public void loadFrom(InputSaveObject sObj) throws SaveObjectException {
		Object [] tabObj = sObj.getNextTab ("tab");
		tab = new SavableMapEntry[tabObj.length];
		for (int i=0;i<tabObj.length;i++)
			tab[i] = (SavableMapEntry) tabObj[i];
	}
	
	/**
	 * Cette classe permet de sauvegarder des couples <Key, Value>
	 *  
	 * @author Benjamin LE BOZEC
	 *
	 */
	public static class SavableMapEntry implements Savable {
		
		private Object key;
		private Object value;

		/**
		 * @param entry
		 */
		public SavableMapEntry (Map.Entry entry) {
			key = entry.getKey();
			value = entry.getValue();
		}
		
		/**
		 * Constructeur par dfaut
		 *
		 */
		public SavableMapEntry  () { }
		
		/**
		 * Accesseur 
		 * @return la cl 'key'
		 */
		public Object getKey () { return key; }
		
		/**
		 * Accesseur
		 * @return la valeur 'Value'
		 */
		public Object getValue () { return value;}
		
		/* (non-Javadoc)
		 * @see think.util.Savable#saveInto(think.util.OutputSaveObject)
		 */
		public void saveInto(OutputSaveObject sObj) throws SaveObjectException {
			sObj.add("key", key);
			sObj.add("value", value);
		}
		
		/* (non-Javadoc)
		 * @see think.util.Savable#loadFrom(think.util.InputSaveObject)
		 */
		public void loadFrom(InputSaveObject sObj) throws SaveObjectException {
			key = sObj.getNext ("key");
			value = sObj.getNext ("value");
		}

		
	}
	
}
