/*
 * Created on 23 fvr. 2005
 *
 */
package util.saveObject;

import org.xml.sax.*;
import java.util.*;

/**
 * Cette classe intervient dans la restitution d'objets enregistrs au format Xml 
 * l'aide de la classe OutputSaveObject.<br>
 * C'est elle qui reconstitue l'arborescence des "ElemSaveObject"
 *   
 * @author Benjamin LE BOZEC
 */
class XmlHandler extends HandlerBase {

	private InputSaveObject currentSaveObj;
	private LinkedList stack;
	private ElemSaveObject currentItem;
	
	/**
	 * Cration d'un XmlHandler
	 * @param sObj l'objet InputSaveObject auquel doit etre associ la classe
	 */
	protected XmlHandler (InputSaveObject sObj) {
		currentSaveObj = sObj;
		stack = new LinkedList ();
		currentItem = null;
	}

	/**
	 * Lecture d'une balise d'ouverture
	 */
	public void startElement (String name, AttributeList attributes) throws SAXException  {
		Class c;
		String className;
		
		if (name.equals("java")) {
			// TODO Verifier version, etc...
		} else {
			if (name.equals ("Object"))
				className = getAttributeValue ("class", attributes);
			else
				className = "java.lang." + name;
			try {
				c = Class.forName (className);
			} catch (ClassNotFoundException e) {
				throw new SAXException ("La classe " + className + " n'existe pas");
			}
			if (SaveObject.isPrimitive(c)) { 
				currentItem = currentSaveObj.createNewEmptyItem();
				currentItem.className = className;
				currentItem.name = getAttributeValue ("name", attributes);
				currentItem.reference = getAttributeValue ("ref", attributes);
				currentItem.value = "";
			} else {
				currentItem = currentSaveObj.createNewItem();
				currentItem.className = className;
				currentItem.name = getAttributeValue ("name", attributes);
				currentItem.reference = getAttributeValue ("ref", attributes);
				stack.addFirst (currentSaveObj);
				currentSaveObj = (InputSaveObject) currentItem.value;
				currentItem = null;
			}
		}
	}
	
	/**
	 * Lecture d'une balise de fermeture
	 */
	public void endElement (String name) throws SAXException {
		
		if (name.equals("java")) {
			
		} else {
			if (currentItem!=null) {
				if (!currentItem.className.equals(name))
					if (!currentItem.className.equals("java.lang."+name))
						throw new SAXException ("prob de balise");
				currentItem=null;
			} else {
				currentSaveObj = (InputSaveObject) stack.removeFirst();
			}
		}
	}

	/**
	 * Lecture du contenu d'une balise
	 */
	public void characters (char ch[], int start, int length) throws SAXException {
		if (currentItem!=null) {
			String content = new String (ch, start, length);
			if (currentItem.value == null)
				currentItem.value = content;
			else
				currentItem.value = ((String) currentItem.value) + content;
		}
    }

	/**
	 * Retourne la valeur de l'attribut qui a pour nom "name" dans la liste "attributes"
	 * @param name Le nom de l'attribut dont la valeur doit tre retourne
	 * @param attributes La liste contenant les attributs
	 * @return La valeur de l'attribut si "name" a t trouv, null sinon 
	 */
	public static String getAttributeValue (String name, AttributeList attributes) {
		for (int i=0;i<attributes.getLength();i++)
			if (attributes.getName(i).equals(name)) 
				return attributes.getValue (i);
		return null;
	}

}
