/*
 * Created on 23 fvr. 2005
 *
 */
package util.saveObject;

import java.io.*;
import java.util.*;

/**
 * Cette classe permet d'crire des documents au format XML.
 * 
 * @author Benjamin LE BOZEC
 */

public class XmlWriter {

	private static final String encoding = "UTF-8";

	private OutputStream out;

	private LinkedList balises;

	/**
	 * Cration d'une nouvelle instance de XmlWriter
	 * 
	 * @param out
	 *            flux de sortie dans lequel le document Xml doit etre crit
	 * @throws IOException
	 *             En cas de probleme d'acces au flux
	 */
	public XmlWriter(OutputStream out) throws IOException {
		this.out = out;
		balises = new LinkedList();
	}

	/**
	 * Ecriture de l'entete \ <?xml ......\>
	 * 
	 * @throws IOException
	 */
	public void writeHeader() throws IOException {
		writeln("<?xml version=" + quote("1.0") + " encoding=" + quote("UTF-8")
				+ " ?>\n");
	}

	/**
	 * Ecriture d'une balise sous la forme : \ <name \prop1="value1" ..... \>
	 * 
	 * @param name
	 *            Nom de la balise
	 * @param properties
	 *            Liste des proprits que doit contenir la balise
	 * @param closed
	 *            Boolen indiquant si la balise doit etre ferm ou non
	 * @param ln
	 *            Boolen prcisant si la balise doit tre suivi d'un retour
	 *            chariot ou non
	 * @throws IOException
	 */
	public void writeBalise(String name, XmlPropertyList properties,
			boolean closed, boolean ln) throws IOException {
		String str = "";
		str = "<" + name;

		if (properties != null) {
			for (int i = 0; i < properties.size(); i++)
				str += " " + properties.getProperty(i);
		}

		if (closed) {
			str += "/>";
		} else {
			str += ">";
			balises.addFirst(name);
		}

		if (ln)
			writeln(str);
		else
			writeContent(str);
	}

	/**
	 * Ecriture d'une simple balise \<name\>
	 * 
	 * @param name
	 *            Nom de la balise
	 * @param closed
	 *            Boolen indiquant si la balise doit etre ferme ou non
	 * @param ln
	 *            Boolen indiquant si la balise doit tre suivie d'un retour
	 *            charriot ou nan
	 * @throws IOException
	 */
	public void writeBalise(String name, boolean closed, boolean ln)
			throws IOException {
		writeBalise(name, null, closed, ln);
	}

	/**
	 * Ecriture d'une balise de fermeture. La balise ferme est la dernire
	 * balise crite a l'aide de writeBalise et qui n'tait pas dfinie comme
	 * ferme (boolen closed)
	 * 
	 * @throws IOException
	 */
	public void closeBalise() throws IOException {
		String name = (String) balises.removeFirst();
		String str = "</" + name + ">";
		writeln(str);
	}

	/**
	 * Ecriture de str suivi d'un retour charriot
	 * 
	 * @param str
	 * @throws IOException
	 */
	public void writeln(String str) throws IOException {
		writeContent(str + "\n");
	}

	/**
	 * Ecriture de str
	 * 
	 * @param str
	 * @throws IOException
	 */
	public void writeContent(String str) throws IOException {
		out.write(str.getBytes(encoding));
		out.flush();
	}

	/**
	 * Retourne la chaine "str" entre quotes
	 * 
	 * @param str
	 * @return
	 */
	protected static String quote(String str) {
		return "\"" + str + "\"";
	}

}
